@@ -13,10 +13,13 @@ use version;
 
 use B::Generate ();
 use B::Deparse  ();
+use XSLoader ();
 
-our $VERSION                    = qv(4.3.2);
+our $VERSION                    = qv(5.0.0);
 our $DEBUG;
 
+XSLoader::load("sealed", $VERSION);
+
 my %valid_attrs                 = (sealed => 1);
 my $p_obj                       = B::svref_2object(sub {&tweak});
 
@@ -25,8 +28,8 @@ my $gv_op                       = $p_obj->START->next->next;
 
 my @replaced_methops;
 
-sub tweak ($\@\@\@$\%) {
-  my ($op, $lexical_varnames, $pads, $op_stack, $cv_obj, $processed_op) = @_;
+sub tweak ($\@\@\@$$\%) {
+  my ($op, $lexical_varnames, $pads, $op_stack, $cv_obj, $pad_names, $processed_op) = @_;
   my $tweaked                   = 0;
 
   if (${$op->next} and $op->next->name eq "padsv") {
@@ -69,20 +72,17 @@ sub tweak ($\@\@\@$\%) {
           if $DEBUG;
         my $method              = $class->can($method_name)
           or die __PACKAGE__ . ": invalid lookup: $class->$method_name - did you forget to 'use $class' first?";
-
         # replace $methop
 
-        my $old_pad = B::cv_pad($cv_obj);
+        my $old_pad             = B::cv_pad($cv_obj);
         my $gv                  = B::GVOP->new($gv_op->name, $gv_op->flags, $method);
-        B::cv_pad($old_pad);
         $gv->next($methop->next);
         $gv->sibparent($methop->sibparent);
-        push @replaced_methops, $methop;
+        push @replaced_methops, [$methop, $pad_names];
         $op->next($gv);
-	# $op->sibparent($gv);
-	# $methop->refcnt_dec if $methop->can("refcnt_dec");
+        B::cv_pad($old_pad);
         $$processed_op{$$_}++ for $op, $gv, $methop;
-
+        _bump_xpadnl_max($pad_names, 1);
         if (ref($gv) eq "B::PADOP") {
           # answer the prayer, by reusing the $targ from the (passed) target pads
           $gv->padix($targ);
@@ -124,7 +124,7 @@ sub MODIFY_CODE_ATTRIBUTES {
       $op->dump if defined $DEBUG and $DEBUG eq 'dump';
 
       if ($op->name eq "pushmark") {
-	$tweaked               += eval {tweak $op, @lexical_varnames, @pads, @op_stack, $cv_obj, %processed_op};
+	$tweaked               += eval {tweak $op, @lexical_varnames, @pads, @op_stack, $cv_obj, $pad_names, %processed_op};
         warn __PACKAGE__ . ": tweak() aborted: $@" if $@;
       }
       elsif (ref($op) eq "B::PMOP") {
@@ -146,9 +146,7 @@ sub MODIFY_CODE_ATTRIBUTES {
       eval {warn B::Deparse->new->coderef2text($rv), "\n"};
       warn "B::Deparse: coderef2text() aborted: $@" if $@;
     }
-
   }
-
   return grep !$valid_attrs{+lc}, @attrs;
 }
 
@@ -191,6 +189,8 @@ For example, any "branching" done in the target method's argument
 stack, eg by using the '?:' ternary operator, will break this logic
 (pushmark ops are processed linearly, by $op->next walking, in tweak()).
 
+Also, :Sealed subs currently aren't reentrant without a custom patch to Perl's pad.c, so don't do that.
+
 =head2 Compiling perl v5.30+ for functional mod_perl2 w/ithreads and httpd 2.4.x w/event mpm
 
     % ./Configure -Uusemymalloc -Duseshrplib -Dusedtrace -Duseithreads -des && make -j$(nproc) && sudo make -j$(nproc) install
